@@ -27,8 +27,6 @@ extern char *optarg;
 #include "uuid/uuid.h"
 #include "e2p/e2p.h"
 
-#include "ext2fs/nls.h"
-
 static FILE *pager;
 
 static void htree_dump_leaf_node(ext2_filsys fs, ext2_ino_t ino,
@@ -138,7 +136,7 @@ static void htree_dump_int_block(ext2_filsys fs, ext2_ino_t ino,
 static void htree_dump_int_node(ext2_filsys fs, ext2_ino_t ino,
 				struct ext2_inode *inode,
 				struct ext2_dx_root_info * rootnode,
-				struct ext2_dx_entry *ent,
+				struct ext2_dx_entry *ent, __u32 crc,
 				char *buf, int level)
 {
 	struct ext2_dx_countlimit	dx_countlimit;
@@ -162,8 +160,11 @@ static void htree_dump_int_node(ext2_filsys fs, ext2_ino_t ino,
 	if (ext2fs_has_feature_metadata_csum(fs->super) &&
 	    remainder == sizeof(struct ext2_dx_tail)) {
 		tail = (struct ext2_dx_tail *)(ent + limit);
-		fprintf(pager, "Checksum: 0x%08x\n",
+		fprintf(pager, "Checksum: 0x%08x",
 			ext2fs_le32_to_cpu(tail->dt_checksum));
+		if (tail->dt_checksum != crc)
+			fprintf(pager, " --- EXPECTED: 0x%08x", crc);
+		fputc('\n', pager);
 	}
 
 	for (i=0; i < count; i++) {
@@ -201,6 +202,7 @@ static void htree_dump_int_block(ext2_filsys fs, ext2_ino_t ino,
 	char		*cbuf;
 	errcode_t	errcode;
 	blk64_t		pblk;
+	__u32		crc;
 
 	cbuf = malloc(fs->blocksize);
 	if (!cbuf) {
@@ -218,13 +220,21 @@ static void htree_dump_int_block(ext2_filsys fs, ext2_ino_t ino,
 	errcode = io_channel_read_blk64(current_fs->io, pblk, 1, buf);
 	if (errcode) {
 		com_err("htree_dump_int_block", errcode,
-			"while 	reading block %llu\n", blk);
+			"while reading block %llu\n", blk);
 		goto errout;
 	}
 
+	errcode = ext2fs_dx_csum(current_fs, ino,
+				 (struct ext2_dir_entry *) buf, &crc, NULL);
+	if (errcode) {
+		com_err("htree_dump_int_block", errcode,
+			"while calculating checksum for logical block %llu\n",
+			(unsigned long long) blk);
+		crc = (unsigned int) -1;
+	}
 	htree_dump_int_node(fs, ino, inode, rootnode,
 			    (struct ext2_dx_entry *) (buf+8),
-			    cbuf, level);
+			    crc, cbuf, level);
 errout:
 	free(cbuf);
 }
@@ -241,6 +251,7 @@ void do_htree_dump(int argc, char *argv[], int sci_idx EXT2FS_ATTR((unused)),
 	struct 		ext2_dx_root_info  *rootnode;
 	struct 		ext2_dx_entry *ent;
 	errcode_t	errcode;
+	__u32		crc;
 
 	if (check_fs_open(argv[0]))
 		return;
@@ -290,12 +301,19 @@ void do_htree_dump(int argc, char *argv[], int sci_idx EXT2FS_ATTR((unused)),
 	fprintf(pager, "\t Hash Version: %d\n", rootnode->hash_version);
 	fprintf(pager, "\t Info length: %d\n", rootnode->info_length);
 	fprintf(pager, "\t Indirect levels: %d\n", rootnode->indirect_levels);
-	fprintf(pager, "\t Flags: %d\n", rootnode->unused_flags);
+	fprintf(pager, "\t Flags: %#x\n", rootnode->unused_flags);
 
 	ent = (struct ext2_dx_entry *)
 		((char *)rootnode + rootnode->info_length);
 
-	htree_dump_int_node(current_fs, ino, &inode, rootnode, ent,
+	errcode = ext2fs_dx_csum(current_fs, ino,
+				 (struct ext2_dir_entry *) buf, &crc, NULL);
+	if (errcode) {
+		com_err("htree_dump_int_block", errcode,
+			"while calculating checksum for htree root\n");
+		crc = (unsigned int) -1;
+	}
+	htree_dump_int_node(current_fs, ino, &inode, rootnode, ent, crc,
 			    buf + current_fs->blocksize,
 			    rootnode->indirect_levels);
 
@@ -310,17 +328,18 @@ errout:
 void do_dx_hash(int argc, char *argv[], int sci_idx EXT2FS_ATTR((unused)),
 		void *infop EXT2FS_ATTR((unused)))
 {
-	ext2_dirhash_t hash, minor_hash, hash_flags;
+	ext2_dirhash_t hash, minor_hash;
 	errcode_t	err;
 	int		c;
 	int		hash_version = 0;
 	__u32		hash_seed[4];
-	const struct nls_table *encoding;
+	int		hash_flags = 0;
+	const struct ext2fs_nls_table *encoding = NULL;
 
 	hash_seed[0] = hash_seed[1] = hash_seed[2] = hash_seed[3] = 0;
 
 	reset_getopt();
-	while ((c = getopt (argc, argv, "h:s:")) != EOF) {
+	while ((c = getopt(argc, argv, "h:s:ce:")) != EOF) {
 		switch (c) {
 		case 'h':
 			hash_version = e2p_string2hash(optarg);
@@ -335,14 +354,16 @@ void do_dx_hash(int argc, char *argv[], int sci_idx EXT2FS_ATTR((unused)),
 			}
 			break;
 		case 'c':
-			hash_flags = EXT4_CASEFOLD_FL;
+			hash_flags |= EXT4_CASEFOLD_FL;
 			break;
 		case 'e':
-			encoding = nls_load_table(e2p_str2encoding(optarg));
-			if (!encoding)
+			encoding = ext2fs_load_nls_table(e2p_str2encoding(optarg));
+			if (!encoding) {
 				fprintf(stderr, "Invalid encoding: %s\n",
 					optarg);
 				return;
+			}
+			break;
 		default:
 			goto print_usage;
 		}
