@@ -34,7 +34,8 @@ void ext2fs_init_csum_seed(ext2_filsys fs)
 {
 	if (ext2fs_has_feature_csum_seed(fs->super))
 		fs->csum_seed = fs->super->s_checksum_seed;
-	else if (ext2fs_has_feature_metadata_csum(fs->super))
+	else if (ext2fs_has_feature_metadata_csum(fs->super) ||
+		 ext2fs_has_feature_ea_inode(fs->super))
 		fs->csum_seed = ext2fs_crc32c_le(~0, fs->super->s_uuid,
 						 sizeof(fs->super->s_uuid));
 }
@@ -265,14 +266,15 @@ static errcode_t __get_dirent_tail(ext2_filsys fs,
 	d = dirent;
 	top = EXT2_DIRENT_TAIL(dirent, fs->blocksize);
 
-	rec_len = translate(d->rec_len);
-	while (rec_len && !(rec_len & 0x3)) {
-		d = (struct ext2_dir_entry *)(((char *)d) + rec_len);
-		if ((void *)d >= top)
-			break;
+	while ((void *) d < top) {
 		rec_len = translate(d->rec_len);
+		if ((rec_len < 8) || (rec_len & 0x03))
+			return EXT2_ET_DIR_CORRUPTED;
+		d = (struct ext2_dir_entry *)(((char *)d) + rec_len);
 	}
 
+	if ((char *)d > ((char *)dirent + fs->blocksize))
+			return EXT2_ET_DIR_CORRUPTED;
 	if (d != top)
 		return EXT2_ET_DIR_NO_SPACE_FOR_CSUM;
 
@@ -289,7 +291,8 @@ static errcode_t __get_dirent_tail(ext2_filsys fs,
 
 int ext2fs_dirent_has_tail(ext2_filsys fs, struct ext2_dir_entry *dirent)
 {
-	return __get_dirent_tail(fs, dirent, NULL, 0) == 0;
+	return __get_dirent_tail(fs, dirent, NULL, 0) !=
+		EXT2_ET_DIR_NO_SPACE_FOR_CSUM;
 }
 
 static errcode_t ext2fs_dirent_csum(ext2_filsys fs, ext2_ino_t inum,
@@ -357,20 +360,31 @@ static errcode_t ext2fs_dirent_csum_set(ext2_filsys fs, ext2_ino_t inum,
 	return 0;
 }
 
-static errcode_t ext2fs_dx_csum(ext2_filsys fs, ext2_ino_t inum,
-				struct ext2_dir_entry *dirent,
-				__u32 *crc, int count_offset, int count,
-				struct ext2_dx_tail *t)
+errcode_t ext2fs_dx_csum(ext2_filsys fs, ext2_ino_t inum,
+			 struct ext2_dir_entry *dirent,
+			 __u32 *crc, struct ext2_dx_tail **ret_t)
 {
 	errcode_t retval;
 	char *buf = (char *)dirent;
 	int size;
-	__u32 old_csum, gen;
+	__u32 gen, dummy_csum = 0;
 	struct ext2_inode inode;
+	struct ext2_dx_tail *t;
+	struct ext2_dx_countlimit *c;
+	int count_offset, limit, count;
+
+	retval = __get_dx_countlimit(fs, dirent, &c, &count_offset, 1);
+	if (retval)
+		return retval;
+	limit = ext2fs_le16_to_cpu(c->limit);
+	count = ext2fs_le16_to_cpu(c->count);
+	if (count_offset + (limit * sizeof(struct ext2_dx_entry)) >
+	    fs->blocksize - sizeof(struct ext2_dx_tail))
+		return EXT2_ET_DIR_NO_SPACE_FOR_CSUM;
+	/* htree structs are accessed in LE order */
+	t = (struct ext2_dx_tail *)(((struct ext2_dx_entry *)c) + limit);
 
 	size = count_offset + (count * sizeof(struct ext2_dx_entry));
-	old_csum = t->dt_checksum;
-	t->dt_checksum = 0;
 
 	retval = ext2fs_read_inode(fs, inum, &inode);
 	if (retval)
@@ -382,10 +396,11 @@ static errcode_t ext2fs_dx_csum(ext2_filsys fs, ext2_ino_t inum,
 				sizeof(inum));
 	*crc = ext2fs_crc32c_le(*crc, (unsigned char *)&gen, sizeof(gen));
 	*crc = ext2fs_crc32c_le(*crc, (unsigned char *)buf, size);
-	*crc = ext2fs_crc32c_le(*crc, (unsigned char *)t,
-				sizeof(struct ext2_dx_tail));
-	t->dt_checksum = old_csum;
+	*crc = ext2fs_crc32c_le(*crc, (unsigned char *)t, 4);
+	*crc = ext2fs_crc32c_le(*crc, (unsigned char *)&dummy_csum, 4);
 
+	if (ret_t)
+		*ret_t = t;
 	return 0;
 }
 
@@ -394,22 +409,9 @@ static int ext2fs_dx_csum_verify(ext2_filsys fs, ext2_ino_t inum,
 {
 	__u32 calculated;
 	errcode_t retval;
-	struct ext2_dx_countlimit *c;
 	struct ext2_dx_tail *t;
-	int count_offset, limit, count;
 
-	retval = __get_dx_countlimit(fs, dirent, &c, &count_offset, 1);
-	if (retval)
-		return 1;
-	limit = ext2fs_le16_to_cpu(c->limit);
-	count = ext2fs_le16_to_cpu(c->count);
-	if (count_offset + (limit * sizeof(struct ext2_dx_entry)) >
-	    fs->blocksize - sizeof(struct ext2_dx_tail))
-		return 0;
-	/* htree structs are accessed in LE order */
-	t = (struct ext2_dx_tail *)(((struct ext2_dx_entry *)c) + limit);
-	retval = ext2fs_dx_csum(fs, inum, dirent, &calculated, count_offset,
-				count, t);
+	retval = ext2fs_dx_csum(fs, inum, dirent, &calculated, &t);
 	if (retval)
 		return 0;
 
@@ -421,22 +423,9 @@ static errcode_t ext2fs_dx_csum_set(ext2_filsys fs, ext2_ino_t inum,
 {
 	__u32 crc;
 	errcode_t retval = 0;
-	struct ext2_dx_countlimit *c;
 	struct ext2_dx_tail *t;
-	int count_offset, limit, count;
 
-	retval = __get_dx_countlimit(fs, dirent, &c, &count_offset, 1);
-	if (retval)
-		return retval;
-	limit = ext2fs_le16_to_cpu(c->limit);
-	count = ext2fs_le16_to_cpu(c->count);
-	if (count_offset + (limit * sizeof(struct ext2_dx_entry)) >
-	    fs->blocksize - sizeof(struct ext2_dx_tail))
-		return EXT2_ET_DIR_NO_SPACE_FOR_CSUM;
-	t = (struct ext2_dx_tail *)(((struct ext2_dx_entry *)c) + limit);
-
-	/* htree structs are accessed in LE order */
-	retval = ext2fs_dx_csum(fs, inum, dirent, &crc, count_offset, count, t);
+	retval = ext2fs_dx_csum(fs, inum, dirent, &crc, &t);
 	if (retval)
 		return retval;
 	t->dt_checksum = ext2fs_cpu_to_le32(crc);
@@ -562,7 +551,7 @@ int ext2fs_inode_bitmap_csum_verify(ext2_filsys fs, dgrp_t group,
 	provided = gdp->bg_inode_bitmap_csum_lo;
 	calculated = ext2fs_crc32c_le(fs->csum_seed, (unsigned char *)bitmap,
 				      size);
-	if (fs->super->s_desc_size >= EXT4_BG_INODE_BITMAP_CSUM_HI_END)
+	if (EXT2_DESC_SIZE(fs->super) >= EXT4_BG_INODE_BITMAP_CSUM_HI_END)
 		provided |= (__u32)gdp->bg_inode_bitmap_csum_hi << 16;
 	else
 		calculated &= 0xFFFF;
@@ -582,7 +571,7 @@ errcode_t ext2fs_inode_bitmap_csum_set(ext2_filsys fs, dgrp_t group,
 
 	crc = ext2fs_crc32c_le(fs->csum_seed, (unsigned char *)bitmap, size);
 	gdp->bg_inode_bitmap_csum_lo = crc & 0xFFFF;
-	if (fs->super->s_desc_size >= EXT4_BG_INODE_BITMAP_CSUM_HI_END)
+	if (EXT2_DESC_SIZE(fs->super) >= EXT4_BG_INODE_BITMAP_CSUM_HI_END)
 		gdp->bg_inode_bitmap_csum_hi = crc >> 16;
 
 	return 0;
@@ -600,7 +589,7 @@ int ext2fs_block_bitmap_csum_verify(ext2_filsys fs, dgrp_t group,
 	provided = gdp->bg_block_bitmap_csum_lo;
 	calculated = ext2fs_crc32c_le(fs->csum_seed, (unsigned char *)bitmap,
 				      size);
-	if (fs->super->s_desc_size >= EXT4_BG_BLOCK_BITMAP_CSUM_HI_LOCATION)
+	if (EXT2_DESC_SIZE(fs->super) >= EXT4_BG_BLOCK_BITMAP_CSUM_HI_LOCATION)
 		provided |= (__u32)gdp->bg_block_bitmap_csum_hi << 16;
 	else
 		calculated &= 0xFFFF;
@@ -620,7 +609,7 @@ errcode_t ext2fs_block_bitmap_csum_set(ext2_filsys fs, dgrp_t group,
 
 	crc = ext2fs_crc32c_le(fs->csum_seed, (unsigned char *)bitmap, size);
 	gdp->bg_block_bitmap_csum_lo = crc & 0xFFFF;
-	if (fs->super->s_desc_size >= EXT4_BG_BLOCK_BITMAP_CSUM_HI_LOCATION)
+	if (EXT2_DESC_SIZE(fs->super) >= EXT4_BG_BLOCK_BITMAP_CSUM_HI_LOCATION)
 		gdp->bg_block_bitmap_csum_hi = crc >> 16;
 
 	return 0;
